﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MemoryStream file = new MemoryStream();
        Ellipse[,] tiles = new Ellipse[8, 8];
        List<List<int>> dataForFill = new List<List<int>>();
        int[] lastFills = new int[8];
        int Score;
        public int score
        {
            get
            {
                return Score;
            }
            set
            {
                Score = value;
                ScoreBlock.Text = Score.ToString();
            }
        }

        public MainWindow()
        {
            InitializeComponent();
            for (int i = 0; i < 8; i++)
            {
                dataForFill.Add(new List<int>());
            }
        }

        private void Grid_Loaded(object sender, RoutedEventArgs e)
        {

        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.ShowDialog();
            file.Flush();
            using (var a = fileDialog.OpenFile())
            {
                a.CopyTo(file);
            }
            LoadData();
        }

        void LoadData()
        {
            gameGrid.Children.Clear();
            var data = Encoding.GetEncoding(1252).GetString(file.ToArray());
            int number;
            for (int row = 0; row < 8; row++)
            {
                for (int column = 0; column < 8; column++)
                {
                    if (int.TryParse(data[row * 10 + column].ToString(), out number))
                    {
                        Border selector = new Border();
                        selector.BorderThickness = new Thickness(2);
                        selector.SetValue(Grid.RowProperty, row);
                        selector.SetValue(Grid.ColumnProperty, column);
                        selector.BorderBrush = gameGrid.Background;
                        Ellipse block = new Ellipse();
                        block.MouseLeftButtonDown += Block_MouseLeftButtonDown;
                        block.MouseRightButtonDown += Block_MouseRightButtonDown;
                        switch (number)
                        {
                            case 1:
                                block.Fill = new SolidColorBrush(Colors.Red);
                                break;
                            case 2:
                                block.Fill = new SolidColorBrush(Colors.Green);
                                break;
                            case 3:
                                block.Fill = new SolidColorBrush(Colors.Blue);
                                break;
                            case 4:
                                block.Fill = new SolidColorBrush(Colors.White);
                                break;
                            case 5:
                                block.Fill = new SolidColorBrush(Colors.Yellow);
                                break;
                            case 6:
                                block.Fill = new SolidColorBrush(Colors.Violet);
                                break;
                            default:
                                block.Fill = new SolidColorBrush(Colors.Red);
                                break;
                        }
                        selector.Child = block;
                        gameGrid.Children.Add(selector);
                        tiles[column, row] = block;
                    }
                }
            }
            int index = 80;
            int columne = 0;
            dataForFill[columne].Clear();
            while (index < data.Length && (int.TryParse(data[index].ToString(), out number) || data[index] == '\r' || data[index] == '\n'))
            {
                char check = data[index];
                if (data[index] == '\r')
                {
                    columne++;
                    if (columne != 8)
                        dataForFill[columne].Clear();
                }
                else if (data[index] != '\n')
                {
                    dataForFill[columne].Add(number);
                }
                index++;
            }
        }

        private void Block_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            //ScoreBlock.Text = "X:" + ((int)(sender as Ellipse).Parent.GetValue(Grid.ColumnProperty)).ToString() + ",Y:" + ((int)(sender as Ellipse).Parent.GetValue(Grid.RowProperty)).ToString();
        }

        Ellipse selected;
        private void Block_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            var clicked = (sender as Ellipse);
            if (selected == null)
            {
                selected = clicked;
                (selected.Parent as Border).BorderBrush = new SolidColorBrush(Colors.Gold);
            }
            else if (selected.Fill != clicked.Fill)
            {

                Vector2 location = new Vector2((int)clicked.Parent.GetValue(Grid.ColumnProperty), (int)clicked.Parent.GetValue(Grid.RowProperty));
                Vector2 locationSwitched = new Vector2((int)selected.Parent.GetValue(Grid.ColumnProperty), (int)selected.Parent.GetValue(Grid.RowProperty));
                if (Math.Abs(location.x - locationSwitched.x) <= 1 && locationSwitched.y - location.y == 0 ||
                    Math.Abs(locationSwitched.y - location.y) <= 1 && location.x - locationSwitched.x == 0)
                {

                    Brush meta = clicked.Fill;
                    clicked.Fill = selected.Fill;
                    selected.Fill = meta;
                    checkAllSurroundings(checkSurroundings(location.x, location.y));
                    checkAllSurroundings(checkSurroundings(locationSwitched.x, locationSwitched.y));
                    if (DestroyLocations.Count != 0)
                    {
                        DestroyLocations.Add(location);
                        DestroyBlocks();
                    }
                    else
                    {
                        meta = clicked.Fill;
                        clicked.Fill = selected.Fill;
                        selected.Fill = meta;
                    }
                    (selected.Parent as Border).BorderBrush = gameGrid.Background;
                    selected = null;
                }
            }
        }
        List<Vector2> DestroyLocations = new List<Vector2>();
        void DestroyBlocks()
        {
            foreach (var item in DestroyLocations)
            {
                tiles[item.x, item.y].Fill = new SolidColorBrush(Colors.Black);
            }
            DestroyLocations.Clear();
        }

        void switchTiles(Vector2 dir, Vector2 tile)
        {
            var now = tiles[tile.x, tile.y].Fill;
            tile.x += dir.x;
            tile.y += dir.y;
            var after = tiles[tile.x, tile.y].Fill;
            var meta = now;
            now = after;
            after = now;
        }

        void resetTile(Vector2 tile)
        {
            var now = tiles[tile.x, tile.y];
            now.Fill = GetNextBlock(tile.x);
        }

        Brush GetNextBlock(int column)
        {
            SolidColorBrush e;
            switch (dataForFill[column][lastFills[column]])
            {
                case 1:
                    e = new SolidColorBrush(Colors.Red); break;
                case 2:
                    e = new SolidColorBrush(Colors.Green); break;
                case 3:
                    e = new SolidColorBrush(Colors.Blue); break;
                case 4:
                    e = new SolidColorBrush(Colors.White); break;
                case 5:
                    e = new SolidColorBrush(Colors.Yellow); break;
                case 6:
                    e = new SolidColorBrush(Colors.Violet); break;
                default:
                    e = new SolidColorBrush(Colors.Red); break;
            }
            lastFills[column]++;
            if (lastFills[column] >= dataForFill[column].Count)
                lastFills[column] = 0;
            return e;
        }

        void checkAllSurroundings(Vector2[] input)
        {
            if (input == null) return;
            foreach (var item in input)
            {
                checkSurroundings(item.x, item.y);
            }
        }

        Vector2[] checkSurroundings(int x, int y)
        {
            List<Vector2> connectedTiles = new List<Vector2>();
            if (tiles[x, y] == null)
                return null;
            var currentTile = tiles[x, y].Fill;
            if (x < 6)
            {
                if (tiles[x + 1, y].Fill.ToString() == currentTile.ToString() && tiles[x + 2, y].Fill.ToString() == currentTile.ToString())
                {
                    score += 3;
                    connectedTiles.AddRange(new Vector2[] { new Vector2(x + 1, y), new Vector2(x + 2, y) });
                }
            }
            if (x > 1)
            {
                if (tiles[x - 1, y].Fill.ToString() == currentTile.ToString() && tiles[x - 2, y].Fill.ToString() == currentTile.ToString())
                {
                    score += 3;
                    connectedTiles.AddRange(new Vector2[] { new Vector2(x - 1, y), new Vector2(x - 2, y) });
                }
            }
            if (y < 6)
            {
                if (tiles[x, y + 1].Fill.ToString() == currentTile.ToString() && tiles[x, y + 2].Fill.ToString() == currentTile.ToString())
                {
                    score += 3;
                    connectedTiles.AddRange(new Vector2[] { new Vector2(x, y + 1), new Vector2(x, y + 2) });
                }
            }
            if (y > 1)
            {
                if (tiles[x, y - 1].Fill.ToString() == currentTile.ToString() && tiles[x, y - 2].Fill.ToString() == currentTile.ToString())
                {
                    score += 3;
                    connectedTiles.AddRange(new Vector2[] { new Vector2(x, y - 1), new Vector2(x, y - 2) });
                }
            }
            if (y > 0 && y < 7)
            {
                if (tiles[x, y - 1].Fill.ToString() == currentTile.ToString() && tiles[x, y + 1].Fill.ToString() == currentTile.ToString())
                {
                    score += 3;
                    connectedTiles.AddRange(new Vector2[] { new Vector2(x, y - 1), new Vector2(x, y + 1) });
                }
            }
            if (x > 0 && x < 7)
            {
                if (tiles[x - 1, y].Fill.ToString() == currentTile.ToString() && tiles[x + 1, y].Fill.ToString() == currentTile.ToString())
                {
                    score += 3;
                    connectedTiles.AddRange(new Vector2[] { new Vector2(x + 1, y), new Vector2(x - 1, y) });
                }
            }

            if (connectedTiles.Count == 0)
                return null;
            foreach (var item in connectedTiles)
            {
                if (!DestroyLocations.Contains(item))
                    DestroyLocations.Add(item);
            }
            if (!DestroyLocations.Contains(new Vector2(x, y)))
                DestroyLocations.Add(new Vector2(x, y));
            return connectedTiles.ToArray();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            ScoreBlock.Text = "0";
            LoadData();
        }

        class Vector2 { public int x; public int y; public Vector2(int _x, int _y) { x = _x; y = _y; } }
    }
}
